==> pangea_sdk-3.3.0.tar.gz: pangea_sdk-3.3.0/pangea/asyncio/services/file_scan.py <==
# Copyright 2022 Pangea Cyber Corporation
# Author: Pangea Cyber Corporation
import io
import logging
from typing import Dict, Optional

import pangea.services.file_scan as m
from pangea.asyncio.request import PangeaRequestAsync
from pangea.request import PangeaConfig
from pangea.response import PangeaResponse, TransferMethod
from pangea.utils import FileUploadParams, get_file_upload_params

from .base import ServiceBaseAsync


class FileScanAsync(ServiceBaseAsync):
    """FileScan service client.

    Provides methods to interact with Pangea FileScan Service:
        https://pangea.cloud/docs/api/embargo

    The following information is needed:
        PANGEA_TOKEN - service token which can be found on the Pangea User
            Console at [https://console.pangea.cloud/project/tokens](https://console.pangea.cloud/project/tokens)

    Examples:
        import os

        # Pangea SDK
        from pangea.config import PangeaConfig
        from pangea.services import FileScan

        PANGEA_TOKEN = os.getenv("PANGEA_TOKEN")

        file_scan_config = PangeaConfig(domain="aws.us.pangea.cloud")

        # Setup Pangea FileScan service
        file_scan = FileScan(token=PANGEA_TOKEN, config=file_scan_config)
    """

    service_name = "file-scan"
    version = "v1"

    async def file_scan(
        self,
        file_path: Optional[str] = None,
        file: Optional[io.BufferedReader] = None,
        verbose: Optional[bool] = None,
        raw: Optional[bool] = None,
        provider: Optional[str] = None,
        sync_call: bool = True,
        transfer_method: TransferMethod = TransferMethod.DIRECT,
    ) -> PangeaResponse[m.FileScanResult]:
        """
        Scan

        Scan a file for malicious content.

        OperationId: file_scan_post_v1_scan

        Args:
            file (io.BufferedReader, optional): file to be scanned (should be opened with read permissions and in binary format)
            file_path (str, optional): filepath to be opened and scanned
            verbose (bool, optional): Echo the API parameters in the response
            raw (bool, optional): Include raw data from this provider
            provider (str, optional): Scan file using this provider
            sync_call (bool, optional): True to wait until server returns a result, False to return immediately and retrieve result asynchronously

        Raises:
            PangeaAPIException: If an API Error happens

        Returns:
            A PangeaResponse where the sanctioned source(s) are in the
                response.result field.  Available response fields can be found
                in our [API Documentation](https://pangea.cloud/docs/api/file-scan).

        Examples:
            try:
                with open("./path/to/file.pdf", "rb") as f:
                    response = client.file_scan(file=f, verbose=True, provider="crowdstrike")
                    print(f"Response: {response.result}")
            except pe.PangeaAPIException as e:
                print(f"Request Error: {e.response.summary}")
                for err in e.errors:
                    print(f"\\t{err.detail} \\n")
        """

        if file or file_path:
            if file_path:
                file = open(file_path, "rb")
            if transfer_method == TransferMethod.DIRECT or transfer_method == TransferMethod.POST_URL:
                params = get_file_upload_params(file)
                crc = params.crc_hex
                sha = params.sha256_hex
                size = params.size
            else:
                crc, sha, size = None, None, None
            files = [("upload", ("filename", file, "application/octet-stream"))]
        else:
            raise ValueError("Need to set file_path or file arguments")

        input = m.FileScanRequest(
            verbose=verbose,
            raw=raw,
            provider=provider,
            transfer_crc32c=crc,
            transfer_sha256=sha,
            transfer_size=size,
            transfer_method=transfer_method,
        )
        data = input.dict(exclude_none=True)
        return await self.request.post("v1/scan", m.FileScanResult, data=data, files=files, poll_result=sync_call)

    async def request_upload_url(
        self,
        transfer_method: TransferMethod = TransferMethod.PUT_URL,
        params: Optional[FileUploadParams] = None,
        verbose: Optional[bool] = None,
        raw: Optional[bool] = None,
        provider: Optional[str] = None,
    ) -> PangeaResponse[m.FileScanResult]:
        input = m.FileScanRequest(
            verbose=verbose,
            raw=raw,
            provider=provider,
            transfer_method=transfer_method,
        )
        if params is not None and (
            transfer_method == TransferMethod.POST_URL or transfer_method == TransferMethod.DIRECT
        ):
            input.transfer_crc32c = params.crc_hex
            input.transfer_sha256 = params.sha256_hex
            input.transfer_size = params.size

        data = input.dict(exclude_none=True)
        return await self.request.request_presigned_url("v1/scan", m.FileScanResult, data=data)


class FileUploaderAsync:
    def __init__(self):
        self.logger = logging.getLogger("pangea")
        self._request: PangeaRequestAsync = PangeaRequestAsync(
            config=PangeaConfig(),
            token="",
            service="FileScanUploader",
            logger=self.logger,
        )

    async def upload_file(
        self,
        url: str,
        file: io.BufferedReader,
        transfer_method: TransferMethod = TransferMethod.PUT_URL,
        file_details: Optional[Dict] = None,
    ):
        if transfer_method == TransferMethod.PUT_URL:
            files = [("file", ("filename", file, "application/octet-stream"))]
            await self._request.put_presigned_url(url=url, files=files)
        elif transfer_method == TransferMethod.POST_URL or transfer_method == TransferMethod.DIRECT:
            files = [("file", ("filename", file, "application/octet-stream"))]
            await self._request.post_presigned_url(url=url, data=file_details, files=files)
        else:
            raise ValueError(f"Transfer method not supported: {transfer_method}")

    async def close(self):
        await self._request.session.close()

==> pangea_sdk-3.7.0.tar.gz: pangea_sdk-3.7.0/pangea/asyncio/services/file_scan.py <==
# Copyright 2022 Pangea Cyber Corporation
# Author: Pangea Cyber Corporation
import io
import logging
from typing import Dict, Optional

import pangea.services.file_scan as m
from pangea.asyncio.request import PangeaRequestAsync
from pangea.request import PangeaConfig
from pangea.response import PangeaResponse, TransferMethod
from pangea.utils import FileUploadParams, get_file_upload_params

from .base import ServiceBaseAsync


class FileScanAsync(ServiceBaseAsync):
    """FileScan service client.

    Provides methods to interact with Pangea FileScan Service:
        https://pangea.cloud/docs/api/embargo

    The following information is needed:
        PANGEA_TOKEN - service token which can be found on the Pangea User
            Console at [https://console.pangea.cloud/project/tokens](https://console.pangea.cloud/project/tokens)

    Examples:
        import os

        # Pangea SDK
        from pangea.config import PangeaConfig
        from pangea.services import FileScan

        PANGEA_TOKEN = os.getenv("PANGEA_TOKEN")

        file_scan_config = PangeaConfig(domain="aws.us.pangea.cloud")

        # Setup Pangea FileScan service
        file_scan = FileScan(token=PANGEA_TOKEN, config=file_scan_config)
    """

    service_name = "file-scan"
    version = "v1"

    async def file_scan(
        self,
        file_path: Optional[str] = None,
        file: Optional[io.BufferedReader] = None,
        verbose: Optional[bool] = None,
        raw: Optional[bool] = None,
        provider: Optional[str] = None,
        sync_call: bool = True,
        transfer_method: TransferMethod = TransferMethod.POST_URL,
        source_url: Optional[str] = None,
    ) -> PangeaResponse[m.FileScanResult]:
        """
        Scan

        Scan a file for malicious content.

        OperationId: file_scan_post_v1_scan

        Args:
            file (io.BufferedReader, optional): file to be scanned (should be opened with read permissions and in binary format)
            file_path (str, optional): filepath to be opened and scanned
            verbose (bool, optional): Echo the API parameters in the response
            raw (bool, optional): Include raw data from this provider
            provider (str, optional): Scan file using this provider
            sync_call (bool, optional): True to wait until server returns a result, False to return immediately and retrieve result asynchronously

        Raises:
            PangeaAPIException: If an API Error happens

        Returns:
            A PangeaResponse where the sanctioned source(s) are in the
                response.result field.  Available response fields can be found
                in our [API Documentation](https://pangea.cloud/docs/api/file-scan).

        Examples:
            try:
                with open("./path/to/file.pdf", "rb") as f:
                    response = client.file_scan(file=f, verbose=True, provider="crowdstrike")
                    print(f"Response: {response.result}")
            except pe.PangeaAPIException as e:
                print(f"Request Error: {e.response.summary}")
                for err in e.errors:
                    print(f"\\t{err.detail} \\n")
        """

        if file or file_path:
            if file_path:
                file = open(file_path, "rb")
            if transfer_method == TransferMethod.POST_URL:
                params = get_file_upload_params(file)  # type: ignore[arg-type]
                crc = params.crc_hex
                sha = params.sha256_hex
                size = params.size
            else:
                crc, sha, size = None, None, None
            files = [("upload", ("filename", file, "application/octet-stream"))]
        else:
            raise ValueError("Need to set file_path or file arguments")

        input = m.FileScanRequest(
            verbose=verbose,
            raw=raw,
            provider=provider,
            crc32c=crc,
            sha256=sha,
            size=size,
            transfer_method=transfer_method,
            source_url=source_url,
        )
        data = input.dict(exclude_none=True)
        return await self.request.post("v1/scan", m.FileScanResult, data=data, files=files, poll_result=sync_call)

    async def request_upload_url(
        self,
        transfer_method: TransferMethod = TransferMethod.PUT_URL,
        params: Optional[FileUploadParams] = None,
        verbose: Optional[bool] = None,
        raw: Optional[bool] = None,
        provider: Optional[str] = None,
    ) -> PangeaResponse[m.FileScanResult]:
        input = m.FileScanRequest(
            verbose=verbose,
            raw=raw,
            provider=provider,
            transfer_method=transfer_method,
        )
        if params is not None and (transfer_method == TransferMethod.POST_URL):
            input.crc32c = params.crc_hex
            input.sha256 = params.sha256_hex
            input.size = params.size

        data = input.dict(exclude_none=True)
        return await self.request.request_presigned_url("v1/scan", m.FileScanResult, data=data)


class FileUploaderAsync:
    def __init__(self) -> None:
        self.logger = logging.getLogger("pangea")
        self._request: PangeaRequestAsync = PangeaRequestAsync(
            config=PangeaConfig(),
            token="",
            service="FileScanUploader",
            logger=self.logger,
        )

    async def upload_file(
        self,
        url: str,
        file: io.BufferedReader,
        transfer_method: TransferMethod = TransferMethod.PUT_URL,
        file_details: Optional[Dict] = None,
    ) -> None:
        if transfer_method == TransferMethod.PUT_URL:
            files = [("file", ("filename", file, "application/octet-stream"))]
            await self._request.put_presigned_url(url=url, files=files)  # type: ignore[arg-type]
        elif transfer_method == TransferMethod.POST_URL:
            files = [("file", ("filename", file, "application/octet-stream"))]
            await self._request.post_presigned_url(url=url, data=file_details, files=files)  # type: ignore[arg-type]
        else:
            raise ValueError(f"Transfer method not supported: {transfer_method}")

    async def close(self) -> None:
        await self._request.session.close()
